/******************************************************************************
*
* PROJECT: Carnegie Mellon Planetary Rover Project
*          Task Control Architecture 
* 
* (c) Copyright 1991 Christopher Fedor and Reid Simmons.  All rights reserved.
* 
* MODULE: datamsg
*
* FILE: datamsg.h
*
* ABSTRACT:
* 
* A datamsg represents the encoded form of a message and is used to perform
* the actual sending and receiving of messages.
*
* REVISION HISTORY
*
*  5-Jul-90 Christopher Fedor, School of Computer Science, CMU
* Added refCount for dealing with datamsg garbage collection.
*
* 13-Jun-90 Christopher Fedor, School of Computer Science, CMU
* Changed parameter names to reflect new design. Replaced issuer, intent, 
* select, object, msgRef with parentRef, intent, classId, dispatchRef, msgRef.
* msgRef is a tca generated msg id on sending, it is primarily used for
* query until a similar class data idea is designed for the module side.
*
* 29-May-90 Christopher Fedor, School of Computer Science, CMU
* Revised for tca 5.x
*
* 26-Apr-89 Christopher Fedor, School of Computer Science, CMU
* Modified to track two seperate data encodings.
*
* 24-Feb-89 Christopher Fedor, School of Computer Science, CMU
* Created.
*
******************************************************************************/

/* The order and number of items in the DataMsg struct is
   CRITCAL to its CORRECT USE - see notes */

/*
  typedef struct _DM {
  int refCount;
  char *classData;
  char *msgData;
  int classTotal;
  int msgTotal;
  int parentRef;
  int intent;
  int classId;
  int dispatchRef;
  int msgRef;
  .
  . <= msgData
  .
  . <= classData
  .
  } DATA_MSG_TYPE, *DATA_MSG_PTR
*/

typedef struct _DM {
  int refCount;
  char *classData;
  char *msgData;
  int classTotal;
  int msgTotal;
  int parentRef;
  int intent;
  int classId;
  int dispatchRef;
  int msgRef;
} DATA_MSG_TYPE, *DATA_MSG_PTR;

/***********************************************************************/

typedef struct {
  int size, allocated;
  DATA_MSG_PTR dataMsg;
} DATA_MSG_BUF_TYPE, *DATA_MSG_BUF_PTR;

extern char *dataMsgDecodeMsg();
extern char *dataMsgDecodeClass();

extern DATA_MSG_PTR dataMsgCreate();
extern DATA_MSG_PTR dataMsgReplaceClassData();

extern TCA_RETURN_STATUS_TYPE dataMsgSend();
extern TCA_RETURN_STATUS_TYPE dataMsgRecv();

extern void dataMsgFree();
extern void dataMsgInitialize();

extern int dataMsgTestMsgData();

extern void dataMsgDisplayStats();

extern TCA_RETURN_STATUS_TYPE writeNBytes();
extern TCA_RETURN_STATUS_TYPE readNBytes();
