/******************************************************************************
*
* PROJECT: Carnegie Mellon Planetary Rover Project
*          Task Control Architecture 
* 
* MODULE: dispatch
*
* FILE: dispatch.h
*
* ABSTRACT:
* 
* A dispatch include file - needs work.
*
* REVISION HISTORY
*
* 26-Jul-91 Reid Simmons, School of Computer Science, CMU
* Added "refCount" field -- the problem is that a dispatch could be freed
* when another module still has a refence to it.
*
* 29-May-91 Christopher Fedor, School of Computer Science, CMU
* Dispatch is getting too messy with block com and class data sillyness.
*
* 16-Apr-91 Christopher Fedor, School of Computer Science, CMU
* Revised to Software Standards.
*
* 15-Apr-91 Reid Simmons, School of Computer Science, CMU
* Added tap field.
*
*  2-Oct-89 Reid Simmons, School of Computer Science, CMU
* Moved "Kill" status to task tree nodes.
*
* 14-Aug-89 Reid Simmons, School of Computer Science, CMU
* Added Status slot; added statuses for killing trees
*
* 28-Apr-89 Christopher Fedor, School of Computer Science, CMU
* Added TreeNode.
*
* 27-Apr-89 Christopher Fedor, School of Computer Science, CMU
* Removed desId - since info is from Hnd. Added TC_ModInfoPtr Org.
*
* 24-Apr-89 Christopher Fedor, School of Computer Science, CMU
* Added Local Id, LocId.
* Changed int MsgId to a Msg and int HndId to Hnd.
*
*  5-Apr-89 Christopher Fedor, School of Computer Science, CMU
* Removed MsgRef. Added RefId. ReImplement using IdTable.
*
* 20-Mar-89 Christopher Fedor, School of Computer Science, CMU
* created.
*
*****************************************************************************/

/* 16-Apr-91: fedor: the number of dispatch status types used in taskTree.c */

#define DISPATCH_STATUS_TOTAL 11 

typedef enum {UnallocatedDispatch, AllocatedDispatch, InactiveDispatch, 
		PendingDispatch, AttendingDispatch, HandledDispatch,
		KilledDispatch, 
		ReplyDispatch, SuccessDispatch, FailureDispatch,
		CentralPendingDispatch
		} DISPATCH_STATUS_TYPE;

typedef struct _DISPATCH {
  int locId;
  int refId;
  int orgId;
  int desId;
  int pRef;

  MSG_PTR msg;
  HND_PTR hnd;
  struct _RESOURCE *resource;

  char *classData;
  TCA_MSG_CLASS_TYPE class;

  char *respData;
  void (*respProc)();

  MODULE_PTR org, des;
  DATA_MSG_PTR msgData, resData;

  struct _TREE_NODE *treeNode;
  struct _DISPATCH *next;

  struct _BLOCK_COM_TYPE *blockCom;

  DISPATCH_STATUS_TYPE status;
  struct _TAP_INFO *tapInfo;
  int refCount;
} DISPATCH_TYPE, *DISPATCH_PTR;

extern void dispatchFree();
extern DISPATCH_PTR dispatchAllocate();

extern char *dispatchDecodeResponse();

extern void dispatchSetMsgData();
extern void dispatchSetResData();

extern DISPATCH_PTR dispatchCopy();

extern void deliverDispatch();
extern void deliverResponse();

extern void dispatchUpdateAndDisplay();

extern void centralReply();
extern void moduleFree();

extern void reserveDispatch();
extern void releaseDispatch();

#define DISPATCH_MSG(dispatch) (dispatch->msg)
#define DISPATCH_MSG_DATA(dispatch) (dispatch->msgData)
#define DISPATCH_RES_DATA(dispatch) (dispatch->resData)
#define DISPATCH_PARENT_REF(dispatch) (dispatch->pRef)
#define DISPATCH_HANDLER_CHOOSEN(dispatch) (dispatch->hnd)
#define DISPATCH_SET_STATUS(newStatus, dispatch) (dispatch->status = newStatus)

extern ID_TABLE_PTR dispatchTable;
#define DISPATCH_FROM_ID(id) (DISPATCH_PTR)idTableItem(id, dispatchTable)
