/******************************************************************************
*
* PROJECT: Carnegie Mellon Planetary Rover Project
*          Task Control Architecture 
* 
* (c) Copyright 1991 Christopher Fedor and Reid Simmons.  All rights reserved.
* 
* MODULE: lex
*
* FILE: lex.c
*
* ABSTRACT:
* 
* primitve lexer
*
* REVISION HISTORY
*
* 13-Feb-89 Christopher Fedor, School of Computer Science, CMU
* Changed to lex the whole string at once.
*
* 10-Feb-89 Christopher Fedor, School of Computer Science, CMU
* Created.
*
* NOTES: 
* 17-Jun-91: fedor:
* Needs to be rewritten to take advantage of abstract data type routines
* and to start looking at memory use. Blah this is a mess!
*
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "basics.h"
#include "lex.h"

#define NEW_TOKEN(Token, Typ) Token = (NEW(TokenType));\
				  Token->Type = Typ;\
                                  Token->Loc = Location;\
				  Token->next = NULL

int Location;
TokenPtr TokenList;

/* Currently not used (RGS: 11/11/92)
char NextChar(s)
char *s;
{
  char c;

  c = s[Location++];

  return c;
}
*/

TokenPtr ReadString(s)
char *s;
{
  char *t2;
  TokenPtr tmp;
  int i, amount;

  for (i=Location;
       (s[i] != ',' && s[i] != '}' && s[i] != '{' && s[i] != '[' 
       && s[i] != ']' && s[i] != ':' && s[i] != ' ' && s[i] != '>'
       && s[i] != '<' && s[i] != '\0');
       i++);

  amount = i-Location;
  NEW_TOKEN(tmp, STR_TOK);
  tmp->value.str = (char *)tcaMalloc(sizeof(char)*(unsigned)(amount+1));
  t2 = s+Location;
  (void)strncpy(tmp->value.str, t2, amount);
  tmp->value.str[amount] = '\0';

  Location += amount;

  return tmp;
}

TokenPtr ReadInt(s)
char *s;
{
  char *t, *t2;
  TokenPtr tmp;
  int i, amount;

  for (i=Location;
       (isdigit(s[i]) && s[i] != ',' && s[i] != '}' && s[i] != '{' 
	&& s[i] != '[' && s[i] != ']' && s[i] != ':' && s[i] != ' ' 
	&& s[i] != '>' && s[i] != '<' && s[i] != '\0');
       i++);

  amount = i-Location;
  NEW_TOKEN(tmp, INT_TOK);
  t = (char *)tcaMalloc(sizeof(char)*(unsigned)(amount+1));
  t2 = s+Location;
  (void)strncpy(t, t2, amount);
  t[amount] = '\0';

#ifndef VXWORKS
  tmp->value.num = atoi(t);
#else
  (void)sscanf(t, "%d", &num);
  tmp->value.num = num;
#endif

  tcaFree(t);

  Location += amount;

  return tmp;
}    

TokenPtr BuildToken(s)
char *s;
{
  TokenPtr Token;

  if (!s) {
    NEW_TOKEN(Token, EOS_TOK);
    return Token;
  }

 Start:
  switch(s[Location]) {
  case ' ':
  case '\t':
  case '\f':
    Location++;
    goto Start;
  case '{':
    NEW_TOKEN(Token, LBRACE_TOK);
    Location++;
    break;
  case '}':
    NEW_TOKEN(Token, RBRACE_TOK);
    Location++;
    break;
  case '[':
    NEW_TOKEN(Token, LBRACK_TOK);
    Location++;
    break;
  case ']':
    NEW_TOKEN(Token, RBRACK_TOK);
    Location++;
    break;
  case '*':
    NEW_TOKEN(Token, STAR_TOK);
    Location++;
    break;
  case '!':
    NEW_TOKEN(Token, BANG_TOK);
    Location++;
    break;
  case ',':
    NEW_TOKEN(Token, COMMA_TOK);
    Location++;
    break;
  case '<':
    NEW_TOKEN(Token, LT_TOK);
    Location++;
    break;
  case '>':
    NEW_TOKEN(Token, GT_TOK);
    Location++;
    break;
  case ':':
    NEW_TOKEN(Token, COLON_TOK);
    Location++;
    break;
  case '\0':
    NEW_TOKEN(Token, EOS_TOK);
    Location++;
    break;
  default:
    if (isdigit(s[Location]))
      Token = ReadInt(s);
    else 
      Token = ReadString(s);
  }

  return Token;
}

PrintToken(Token)
TokenPtr Token;
{
  fprintf(stderr, "Loc: %d: Type: ", Token->Loc);

  switch(Token->Type) {
  case LBRACE_TOK:
    fprintf(stderr, "LBRACE_TOK\n");
    break;
  case RBRACE_TOK:
    fprintf(stderr, "RBRACE_TOK\n");
    break;
  case LBRACK_TOK:
    fprintf(stderr, "LBRACK_TOK\n");
    break;
  case RBRACK_TOK:
    fprintf(stderr, "RBRACK_TOK\n");
    break;
  case STAR_TOK:
    fprintf(stderr, "STAR_TOK\n");
    break;
  case BANG_TOK:
    fprintf(stderr, "BANG_TOK\n");
    break;
  case COMMA_TOK:
    fprintf(stderr, "COMMA_TOK\n");
    break;
  case LT_TOK:
    fprintf(stderr, "LT_TOK\n");
    break;
  case GT_TOK:
    fprintf(stderr, "GT_TOK\n");
    break;
  case COLON_TOK:
    fprintf(stderr, "COLON_TOK\n");
    break;
  case EOS_TOK:
    fprintf(stderr, "EOS_TOK\n");
    break;
  case INT_TOK:
    fprintf(stderr, "INT_TOK: %d\n", Token->value.num);
    break;
  case STR_TOK:
    fprintf(stderr, "STR_TOK: %s\n", Token->value.str);
    break;
  case PRIMITIVE_TOK:
    fprintf(stderr, "PRIMITIVE_TOK: %s\n", Token->value.str);
    break;
  default:
    fprintf(stderr, "Token of Unknown type: %d\n", Token->Type);
    break;
  }
}

LexString(s)
char *s;
{
  TokenPtr Token;

  Location = 0;

  TokenList = BuildToken(s);

  Token = TokenList;
  while (Token->Type != EOS_TOK) {
    Token->next = BuildToken(s);
    Token = Token->next;
  }
}


PrintTokenList(Token)
TokenPtr Token;
{
  TokenPtr tmp;

  tmp = Token;
  while (tmp) {
    PrintToken(tmp);
    tmp = tmp->next;
  }
}

TokenPtr NextToken()
{
  TokenPtr tmp;

  tmp = TokenList;
  TokenList = TokenList->next;

  return tmp;
}

/*
main()
{
  
  LexString("{mumble}");

  PrintTokenList(TokenList);

}
*/

