/******************************************************************************
*
* PROJECT: Carnegie Mellon Planetary Rover Project
*          Task Control Architecture 
* 
* (c) Copyright 1991 Christopher Fedor and Reid Simmons.  All rights reserved.
* 
* MODULE: tca
*
* FILE: msgTap.h
*
* ABSTRACT: Type definitions needed for the "wire tapping" mechanism
*
******************************************************************************/

typedef struct {
  TAP_CONDITION_TYPE condition;
  MSG_PTR listeningMsg;
} TAP_TYPE, *TAP_PTR;

typedef struct _TAP_INFO {
  DISPATCH_PTR tappedDispatch;
  struct _LIST *extraTaps;
} TAP_INFO_TYPE, *TAP_INFO_PTR;

typedef struct {
  TAP_CONDITION_TYPE condition;
  char *tappedMsg;
  char *listeningMsg;
} TAP_MSG_TYPE, *TAP_MSG_PTR;

#define TAP_MSG_INFORM_FORM "{int, string, string}"

typedef struct {
  TAP_CONDITION_TYPE condition;
  TCA_REF_PTR tappedRef;
  char *listeningMsg;
} TAP_REF_TYPE, *TAP_REF_PTR;

#define TAP_REF_INFORM_FORM "{int, TCA_REF_PTR, string}"

extern void tapBeforeHandling();
extern void tapWhileHandling();
extern void tapWhenSent();
extern void tapAfterReplied();
extern void tapAfterHandled();
extern void tapAfterSuccess();
extern void tapAfterFailure();
extern void tapWhenSuccess();
extern void tapWhenFailure();

extern void setUpBeforeTaps();
extern void setUpDuringAndAfterTaps();

#define DISPATCH_TAPS(dispatch) \
  (((dispatch)->msg) ? (dispatch)->msg->tapList : NULL)

#define DISPATCH_EXTRA_TAPS(dispatch) \
  (((dispatch)->tapInfo) ? (dispatch)->tapInfo->extraTaps : NULL)
