/******************************************************************************
*
* PROJECT: Carnegie Mellon Planetary Rover Project
*          Task Control Architecture 
* 
* (c) Copyright 1991 Christopher Fedor and Reid Simmons.  All rights reserved.
* 
* MODULE: idTable
*
* FILE: idtable.h
*
* ABSTRACT:
* 
* Generic dynamic array abstract data type.
*
* REVISION HISTORY
*
* 25-Aug-90 Christopher Fedor, School of Computer Science, CMU
* Revised to Software Standards.
*
* 20-Feb-89 Christopher Fedor, School of Computer Science, CMU
* created.
*
******************************************************************************/

#include <stdio.h>

#include "basics.h"
#include "idtable.h"

ID_TABLE_PTR idTableCreate(name, incSize)
char *name;
int incSize;
{
  ID_TABLE_PTR idTable;

  idTable = NEW(ID_TABLE_TYPE);

  idTable->name = name;
  idTable->incSize = incSize;
  idTable->freeTotal = 0;
  idTable->currentSize = 0;
  idTable->table = NULL;
  idTable->freeList = NULL;

  return idTable;
}


void idTableIncrement(table)
ID_TABLE_PTR table;
{
  int i, newTableSize, currentSize, newFreeIds;
  TELEM_PTR tmp, newTable, oldTable, newFreeList, lastNewFreeBlock;

  newFreeIds = table->incSize;
  currentSize = table->currentSize;

  newTableSize = currentSize+newFreeIds;

  oldTable = table->table;
  newTable = (TELEM_TYPE *)tcaMalloc(sizeof(TELEM_TYPE)*(unsigned)newTableSize);

  i = 0;

  if (oldTable) {
    /* Copy Old Table */
    for (;i<currentSize;i++)
      newTable[i] = oldTable[i];
    tcaFree((char *)oldTable);
  }

  /* Update Free List and Initialize New Blocks */
  newFreeList = NULL;
  lastNewFreeBlock = newTable + i;
  for (;i<newTableSize;i++) {
    tmp = newTable + i;
    tmp->data = NULL;
    tmp->ref = i;
    tmp->next = newFreeList;
    newFreeList = tmp;
  }
  
  /* Adjust Table Entry */
  lastNewFreeBlock->next = table->freeList;
  table->freeList = newFreeList;

  table->freeTotal += newFreeIds;
  table->currentSize = newTableSize;

  table->table = newTable;
}

int idTableInsert(item, table)
char *item;
ID_TABLE_PTR table;
{
  TELEM_PTR tmp;
  
  if (!table->freeList)
    idTableIncrement(table);

  tmp = table->freeList;
  table->freeList = tmp->next;
  tmp->next = NULL;
  table->freeTotal--;

  tmp->data = item;

  return(tmp->ref);
}

char *idTableRemove(id, table)
int id;
ID_TABLE_PTR table;
{
  char *data;
  TELEM_PTR tmp;

  tmp = table->table+id;

  data = tmp->data;
  
  if (!tmp->next) {
    /* Not Already on Free List */
    tmp->data = NULL;
    tmp->next = table->freeList;
    table->freeList = tmp;
    table->freeTotal++;
  }

  return data;
}

void idTablePrintInfo(table)
ID_TABLE_PTR table;
{
  if (table) {
    fprintf(stderr, "idTable: %s\n", table->name);
    fprintf(stderr, "free   : %d\n", table->freeTotal);
    fprintf(stderr, "size   : %d\n", table->currentSize);
    fprintf(stderr, "incSize: %d\n", table->incSize);
  }
  else
    fprintf(stderr, "idTablePrintInfo: NULL Table\n");
}
