/******************************************************************************
*  tplconstr.h
*
*  Partial temporal ordering constraints
*
*  14-Aug-89 Reid Simmons       Added routines for killing task trees
*                               (Cancel_Activation_Message, FreeInterval, and
*                               KillAfterAttending check).
*   2-Aug-89 Reid Simmons       Changed "printf"s to use logging facilities.
*                               Made changes so you can now issue 
*                               commands and goals from a Query.
*  15-May-89 Christopher Fedor  Changed interval type not to make use of
*                               quantity_ptr typedef to minimize include files
*                               needed in modules that do not use qlattice.
*  28-Apr-89 Reid Simmons       Created (based on existing QLattice code)
******************************************************************************/

typedef enum {start_point, end_point} time_types;

typedef enum {handlingInterval, achievementInterval, 
		planningInterval} interval_types;

typedef struct {
  time_types     time;
  interval_types interval;
} time_selection_type, *time_selection_ptr;

typedef struct _interval_type {
  struct _QUANTITY_TYPE *start, *end;
} interval_type, *interval_ptr;

#define DispatchTplConstraints(dispatch) (dispatch)->msgData->dispatchRef

extern void Tpl_Initialize();
extern void Initialize_TaskTreeNode();

extern void AssertHoldingConstraint(); 

extern void RetractHoldingConstraint();

extern void PendingConstraints();
extern void InactiveConstraints();
extern void AttendingConstraints(); 
extern void CompletionConstraints();

extern void Try_To_Activate();

/******************************************************************************
* Free the interval and its associated start and end points.
******************************************************************************/
extern void FreeInterval();
/* interval_ptr Interval; */

extern void addTplConstraints();

extern int Less_Than_Now_Relp();
extern int Relp_Still_Out();
