/*****************************************************************************
*
* PROJECT: Carnegie Mellon Planetary Rover Project
*          Task Control Architecture
*
* (c) Copyright 1991 Christopher Fedor and Reid Simmons.  All rights reserved.
* 
* MODULE: Temporal Constraints
*
* FILE: tplModule.c
*
* ABSTRACT:
* Modules interface for using temporal constraints
*
* REVISION HISTORY:
*
* 12-May-89  Reid Simmons, School of Computer Science, CMU
* Split from tplConstr.c
*
*****************************************************************************/

#ifdef VXWORKS
#include "stdioLib.h"
#else
#include "stdio.h"
#endif

#include "tcaInternal.h"
#include "datamsg.h"

TCA_TIME_POINT_TYPE tcaDefaultTimeGlobal = {NoTime, {NoInterval, 0}};

TCA_TIME_POINT_TYPE tcaStartOf(interval) 
     TCA_INTERVAL_TYPE interval;
{ 
  TCA_TIME_POINT_TYPE startTime;

  startTime.class = StartPoint;
  startTime.interval = interval;
  return startTime;
}

TCA_TIME_POINT_TYPE tcaEndOf(interval) 
     TCA_INTERVAL_TYPE interval;
{ 
  TCA_TIME_POINT_TYPE endTime;

  endTime.class = EndPoint;
  endTime.interval = interval;
  return endTime;
}

TCA_INTERVAL_TYPE tcaHandlingOf(msgRef) 
     TCA_REF_PTR msgRef;
{ 
  TCA_INTERVAL_TYPE handlingInterval;

  handlingInterval.class = HandlingInterval;
  handlingInterval.msgRef = (msgRef) ? msgRef->refId : -1;
  return handlingInterval;
}

TCA_INTERVAL_TYPE tcaPlanningOf(msgRef) 
     TCA_REF_PTR msgRef;
{ 
  TCA_INTERVAL_TYPE planningInterval;

  planningInterval.class = PlanningInterval;
  planningInterval.msgRef = (msgRef) ? msgRef->refId : -1;
  return planningInterval;
}

TCA_INTERVAL_TYPE tcaAchievementOf(msgRef) 
     TCA_REF_PTR msgRef;
{ 
  TCA_INTERVAL_TYPE achievementInterval;

  achievementInterval.class = AchievementInterval;
  achievementInterval.msgRef = (msgRef) ? msgRef->refId : -1;
  return achievementInterval;
}

void tcaTplConstrain(timePoint1, relationship, timePoint2)
     TCA_TIME_POINT_TYPE timePoint1; 
     char *relationship; 
     TCA_TIME_POINT_TYPE timePoint2;
{ 
  TCA_TPL_CONSTRAINT_TYPE tplConstraint;

  tplConstraint.timePoint1 = timePoint1;
  tplConstraint.relationship = relationship;
  tplConstraint.timePoint2 = timePoint2;
  
  (void)tcaInform("tplConstrain", (void *)&tplConstraint);
}
